# generated by datamodel-codegen:
#   filename:  aws-cfn-kafka-admin-provider-schema.json
#   timestamp: 2021-11-16T22:30:02+00:00

from __future__ import annotations

from enum import Enum
from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Extra, Field, constr


class DeletionPolicy(Enum):
    Retain = "Retain"
    Delete = "Delete"


class CompatibilityMode(Enum):
    BACKWARD = "BACKWARD"
    BACKWARD_TRANSITIVE = "BACKWARD_TRANSITIVE"
    FORWARD = "FORWARD"
    FORWARD_TRANSITIVE = "FORWARD_TRANSITIVE"
    FULL = "FULL"
    FULL_TRANSITIVE = "FULL_TRANSITIVE"
    NONE = "NONE"


class SerializerDef(Enum):
    AVRO = "AVRO"
    JSON = "JSON"
    PROTOBUF = "PROTOBUF"


class TopicSchemaDef(BaseModel):
    Serializer: SerializerDef
    Definition: Optional[Union[str, Dict[str, Any]]] = None
    CompatibilityMode: Optional[CompatibilityMode] = None


class BootstrapServers(BaseModel):
    __root__: str = Field(
        ..., description="Endpoint URL of the Kafka cluster in the format hostname:port"
    )


class SecurityProtocol(Enum):
    PLAINTEXT = "PLAINTEXT"
    SSL = "SSL"
    SASL_PLAINTEXT = "SASL_PLAINTEXT"
    SASL_SSL = "SASL_SSL"


class SASLMechanism(Enum):
    PLAIN = "PLAIN"
    GSSAPI = "GSSAPI"
    OAUTHBEARER = "OAUTHBEARER"
    SCRAM_SHA_256 = "SCRAM-SHA-256"
    SCRAM_SHA_512 = "SCRAM-SHA-512"


class SASLUsername(BaseModel):
    __root__: str = Field(..., description="Kafka SASL username for Authentication")


class SASLPassword(BaseModel):
    __root__: str = Field(..., description="Kafka SASL password for Authentication")


class RegistryUrl(BaseModel):
    __root__: str = Field(..., description="Schema registry URL")


class RegistryUsername(BaseModel):
    __root__: str = Field(..., description="Schema registry username")


class RegistryPassword(BaseModel):
    __root__: str = Field(..., description="Schema registry password")


class RegistryUserInfo(BaseModel):
    __root__: str = Field(
        ...,
        description="The username and password together in the form of username:password",
    )


class Name(BaseModel):
    __root__: constr(regex=r"^[a-zA-Z0-9_.-]+$", min_length=1) = Field(
        ..., description="Kafka topic name"
    )


class PartitionsCount(BaseModel):
    __root__: int = Field(
        ..., description="Number of partitions for the new Kafka topic"
    )


class ReplicationFactor(BaseModel):
    __root__: int = Field(..., description="Kafka topic replication factor")


class TopicSchemas(BaseModel):
    Key: Optional[TopicSchemaDef] = None
    Value: Optional[TopicSchemaDef] = None
    Header: Optional[TopicSchemaDef] = None


class PatternType(Enum):
    LITERAL = "LITERAL"
    PREFIXED = "PREFIXED"
    MATCH = "MATCH"


class ResourceType(Enum):
    CLUSTER = "CLUSTER"
    DELEGATION_TOKEN = "DELEGATION_TOKEN"
    GROUP = "GROUP"
    TOPIC = "TOPIC"
    TRANSACTIONAL_ID = "TRANSACTIONAL_ID"


class Action(Enum):
    ALL = "ALL"
    READ = "READ"
    WRITE = "WRITE"
    CREATE = "CREATE"
    DELETE = "DELETE"
    ALTER = "ALTER"
    DESCRIBE = "DESCRIBE"
    CLUSTER_ACTION = "CLUSTER_ACTION"
    DESCRIBE_CONFIGS = "DESCRIBE_CONFIGS"
    ALTER_CONFIGS = "ALTER_CONFIGS"
    IDEMPOTENT_WRITE = "IDEMPOTENT_WRITE"


class Effect(Enum):
    DENY = "DENY"
    ALLOW = "ALLOW"


class PolicyDef(BaseModel):
    class Config:
        extra = Extra.forbid

    Resource: str = Field(..., description="Name of the resource to apply the ACL for")
    PatternType: Optional[PatternType] = Field(
        "LITERAL", description="Pattern type for resource value"
    )
    Principal: str = Field(..., description="Kafka user to apply the ACLs for.")
    ResourceType: ResourceType = Field(
        ..., description="Kafka user to apply the ACLs for."
    )
    Action: Action = Field(..., description="Access action allowed.")
    Effect: Effect = Field(..., description="Effect for the ACL.")
    Host: Optional[str] = Field(
        "*", description="Specify the host for the ACL. Defaults to '*'"
    )


class EwsKafkaSchema(BaseModel):
    RegistryUrl: Optional[RegistryUrl] = None
    RegistryUsername: Optional[RegistryUsername] = None
    RegistryPassword: Optional[RegistryPassword] = None
    RegistryUserInfo: Optional[RegistryUserInfo] = None
    Subject: Optional[str] = None
    Serializer: Optional[SerializerDef] = None
    Definition: Optional[Union[str, Dict[str, Any]]] = None
    CompatibilityMode: Optional[CompatibilityMode] = None
    ServiceToken: Optional[str] = Field(None, description="The Lambda Function ARN")
    PermanentlyDelete: Optional[bool] = Field(
        False,
        description="If set to true, the Schema is set to hard delete. Use carefully",
    )


class EwsKafkaAcl(BaseModel):
    Policies: Optional[List[PolicyDef]] = None


class Schemas(BaseModel):
    FunctionName: Optional[str] = Field(
        None, description="Name or ARN of the Schema Registry function to use"
    )
    RegistryUrl: Optional[str] = None
    RegistryUsername: Optional[RegistryUsername] = None
    RegistryPassword: Optional[RegistryPassword] = None
    RegistryUserInfo: Optional[RegistryUserInfo] = None
    CompatibilityMode: Optional[CompatibilityMode] = None


class EwsKafkaParameters(BaseModel):
    BootstrapServers: Optional[BootstrapServers] = None
    SecurityProtocol: Optional[SecurityProtocol] = None
    SASLMechanism: Optional[SASLMechanism] = None
    SASLUsername: Optional[SASLUsername] = None
    SASLPassword: Optional[SASLPassword] = None
    RegistryUrl: Optional[RegistryUrl] = None
    RegistryUsername: Optional[RegistryUsername] = None
    RegistryPassword: Optional[RegistryPassword] = None
    CompatibilityMode: Optional[CompatibilityMode] = None


class EwsKafkaTopic(BaseModel):
    Name: Name
    PartitionsCount: PartitionsCount
    ReplicationFactor: Optional[ReplicationFactor] = None
    BootstrapServers: Optional[BootstrapServers] = None
    SecurityProtocol: Optional[SecurityProtocol] = None
    SASLMechanism: Optional[SASLMechanism] = None
    SASLUsername: Optional[SASLUsername] = None
    SASLPassword: Optional[SASLPassword] = None
    Schema: Optional[TopicSchemas] = None


class Policies(BaseModel):
    __root__: List[PolicyDef]


class SchemasDef(BaseModel):
    __root__: EwsKafkaSchema


class AclsModel(BaseModel):
    __root__: EwsKafkaAcl


class Topics(BaseModel):
    Topics: Optional[List[EwsKafkaTopic]] = None
    ReplicationFactor: Optional[ReplicationFactor] = None
    FunctionName: Optional[str] = Field(
        None,
        description="Name or ARN of the Lambda function to use for Custom::KafkaTopic",
    )
    DeletionPolicy: Optional[DeletionPolicy] = "Retain"
    ImportExisting: Optional[bool] = Field(
        True,
        description="Whether to import existing topics on Create. Fails if set to false",
    )


class ACLs(BaseModel):
    Policies: Optional[Policies] = None
    FunctionName: Optional[str] = Field(
        None,
        description="Name or ARN of the Lambda function to use for Custom::KafkaACL",
    )


class Model(BaseModel):
    Globals: EwsKafkaParameters
    Topics: Optional[Topics] = None
    ACLs: Optional[ACLs] = None
    Schemas: Optional[Schemas] = None



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2021 John Mille <john@ews-network.net>


"""Main module."""

import json
import re
from copy import deepcopy

import yaml
from compose_x_common.compose_x_common import keyisset, keypresent

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

from troposphere import AWS_NO_VALUE, GetAtt, Ref, Sub, Template

from cfn_kafka_admin.models.admin import *

from .cfn_resources_definitions import KafkaAclPolicy
from .cfn_resources_definitions.custom import KafkaAcl as CACLs
from .cfn_resources_definitions.custom import KafkaTopic as CTopic
from .cfn_resources_definitions.custom import KafkaTopicSchema as CTopicSchema
from .cfn_resources_definitions.resource import KafkaAcl as RACLs
from .cfn_resources_definitions.resource import KafkaTopic as RTopic
from .cfn_resources_definitions.resource import KafkaTopicSchema as RTopicSchema

NONALPHANUM = re.compile(r"([^a-zA-Z0-9]+)")


def merge_topics(final, override, extend_config_only=False):
    """
    Function to override and update settings from override to primary
    Topics are filtered out via the Name property
    :param dict final:
    :param dict override:
    :param extend_config_only: Whether the policies or ACLs can be merged.
    :return: The final merged dict
    :rtype: dict
    """
    if keyisset("Topics", override):
        override_top_topics = Topics.parse_obj(override["Topics"]).dict()
        if extend_config_only:
            # Allows to add the config and ensure that we do not import topics from config
            if keypresent("Topics", override_top_topics):
                del override_top_topics["Topics"]
            final["Topics"].update(override_top_topics)
        elif not extend_config_only and keyisset("Topics", override_top_topics):
            if keyisset("Topics", final["Topics"]):
                existing_topics = deepcopy(final["Topics"]["Topics"])
                override_topics = deepcopy(override_top_topics["Topics"])
                merged_lists = override_topics + existing_topics
                del final["Topics"]["Topics"]
                del override_top_topics["Topics"]
                topics = list({v["Name"]: v for v in merged_lists}.values())
                final["Topics"].update(override_top_topics)
                final["Topics"]["Topics"] = topics
            elif not keypresent("Topics", final["Topics"]):
                override_topics = deepcopy(override_top_topics["Topics"])
                del override_top_topics["Topics"]
                topics = list({v["Name"]: v for v in override_topics}.values())
                final["Topics"].update(override_top_topics)
                final["Topics"]["Topics"] = topics


def merge_acls(final, override, extend_all=False):
    """
    Function to override and update settings from override to primary
    All ACL policy is a dictionary made of simple objects, no key filtering

    :param dict final:
    :param dict override:
    :param extend_all: Whether the policies or ACLs can be merged.
    :return: The final merged dict
    :rtype: dict
    """
    if keyisset("ACLs", override):
        override_acls = ACLs.parse_obj(override["ACLs"]).dict()
        if keypresent("Policies", override_acls) and not extend_all:
            del override_acls["Policies"]
            final["ACLs"].update(override_acls)
        elif keyisset("Policies", override_acls) and extend_all:
            if keyisset("Policies", final["ACLs"]):
                merged_lists = override_acls["Policies"] + final["ACLs"]["Policies"]
            else:
                merged_lists = override_acls["Policies"]
            acls = [dict(y) for y in set(tuple(x.items()) for x in merged_lists)]
            final["ACLs"].update(override_acls)
            final["ACLs"]["Policies"] = acls


def merge_contents(primary, override, extend_all=False):
    """
    Function to override and update settings from override to primary
    :param primary:
    :param override:
    :param extend_all: Whether the policies or ACLs can be merged.
    :return: The final merged dict
    :rtype: dict
    """
    if not isinstance(override, dict):
        raise TypeError(
            "The content of the override file does not match the expected content pattern."
        )
    final = dict(deepcopy(primary))
    if (
        keypresent("Globals", final)
        and keyisset("Globals", override)
        and isinstance(override["Globals"], dict)
    ):
        override_globals = EwsKafkaParameters.parse_obj(override["Globals"])
        final["Globals"].update(override_globals.dict())

    if (
        keypresent("Schemas", final)
        and keyisset("Schemas", override)
        and isinstance(override["Schemas"], dict)
    ):
        override_globals = Schemas.parse_obj(override["Schemas"])
        final["Schemas"].update(override_globals.dict())
    elif (
        not keypresent("Schemas", final)
        and keyisset("Schemas", override)
        and isinstance(override["Schemas"], dict)
    ):
        override_globals = Schemas.parse_obj(override["Schemas"])
        final["Schemas"] = override_globals.dict()

    merge_acls(final, override, extend_all)
    merge_topics(final, override, not extend_all)
    return final


class KafkaStack(object):
    """
    Class to represent the Kafka topics / acls / schemas in CloudFormation.
    """

    def __init__(self, files_paths, config_file_path=None):
        self.model = None
        self.template = Template("Kafka topics-acls-schemas root")
        self.stack = None
        self.topic_class = RTopic
        self.acl_class = RACLs
        self.schemas_r = {}
        self.topics_r = {}
        self.globals_config = {}
        final_content = {"Globals": {}, "Topics": {}, "ACLs": {}}
        for file_path in files_paths:
            if file_path.endswith(".yaml") or file_path.endswith(".yml"):
                with open(file_path, "r") as file_fd:
                    file_content = file_fd.read()
                yaml_content = yaml.load(file_content, Loader=Loader)
                final_content = merge_contents(
                    final_content, yaml_content, extend_all=True
                )
        if config_file_path:
            with open(config_file_path, "r") as override_fd:
                override_content = override_fd.read()
            override_content = yaml.load(override_content, Loader=Loader)
            final_content = merge_contents(final_content, override_content)
        self.model = Model.parse_obj(final_content)

        if not self.model.Topics and not self.model.ACLs:
            raise KeyError("You must define at least one of ACLs or Topics")
        self.set_globals()

    def set_globals(self):
        """
        Method to set the global settings
        """
        self.globals_config.update(
            {
                "BootstrapServers": self.model.Globals.BootstrapServers.__root__,
                "SASLUsername": self.model.Globals.SASLUsername.__root__
                if self.model.Globals.SASLUsername.__root__
                else Ref(AWS_NO_VALUE),
                "SASLPassword": self.model.Globals.SASLPassword.__root__
                if self.model.Globals.SASLPassword.__root__
                else Ref(AWS_NO_VALUE),
                "SASLMechanism": SASLMechanism[
                    self.model.Globals.SASLMechanism.name
                ].value
                if isinstance(self.model.Globals.SASLMechanism, SASLMechanism)
                else self.model.Globals.SASLMechanism,
                "SecurityProtocol": SecurityProtocol[
                    self.model.Globals.SecurityProtocol.name
                ].value
                if isinstance(self.model.Globals.SecurityProtocol, SecurityProtocol)
                else self.model.Globals.SecurityProtocol,
            }
        )

    def add_attribute_schema(
        self,
        topic_name,
        registry_url,
        registry_username,
        registry_password,
        registry_userinfo,
        schema_class,
        attribute,
        subject_suffix,
    ):
        if isinstance(attribute.Definition, str):
            try:
                with open(attribute.Definition, "r") as definition_fd:
                    definition = json.dumps(json.loads(definition_fd.read()))
            except FileNotFoundError:
                print("Failed to load file, using string literal as definition")
                definition = attribute.Definition
        else:
            definition = attribute.Definition
        topic_schema_r = schema_class(
            f"{topic_name}{SerializerDef[attribute.Serializer.name].value}{subject_suffix}Schema",
            SerializeAttribute=subject_suffix,
            Serializer=SerializerDef[attribute.Serializer.name].value,
            Definition=definition,
            Subject=Sub(f"${{{topic_name}.Name}}-{subject_suffix}"),
            RegistryUrl=registry_url,
            RegistryUsername=registry_username
            if isinstance(registry_username, Ref)
            else registry_username.__root__,
            RegistryPassword=registry_password
            if isinstance(registry_password, Ref)
            else registry_password.__root__,
            RegistryUserInfo=registry_userinfo.__root__,
            CompatibilityMode=CompatibilityMode[attribute.CompatibilityMode.name].value,
        )
        if schema_class is CTopicSchema:
            function_name = (
                self.model
                if self.model.Schemas.FunctionName.startswith("arn:aws")
                else Sub(
                    "arn:${AWS::Partition}:lambda:${AWS::Region}:${AWS::AccountId}:function:"
                    f"{self.model.Schemas.FunctionName}"
                )
            )
            setattr(topic_schema_r, "ServiceToken", function_name)
        self.schemas_r[topic_name] = topic_schema_r
        self.template.add_resource(topic_schema_r)

    def add_topic_schema(self, topic_name, schema_definition):
        """
        Method to create a new schema for the given topic

        :param str topic_name:
        :param Schema schema_definition:
        """
        schema_class = RTopicSchema
        if self.model.Schemas and self.model.Schemas.FunctionName:
            schema_class = CTopicSchema
        schema_config = schema_definition.dict()

        if self.model.Schemas and self.model.Schemas.RegistryUrl:
            registry_url = self.model.Schemas.RegistryUrl
        elif keyisset("RegistryUrl", schema_config):
            registry_url = schema_config["RegistryUrl"]
        else:
            raise KeyError(
                "RegistryUrl is not defined in Schema settings nor in Schemas"
            )

        if self.model.Schemas and self.model.Schemas.RegistryUsername:
            registry_username = self.model.Schemas.RegistryUsername
        elif keyisset("RegistryUsername", schema_config):
            registry_username = schema_config["RegistryUsername"]
        else:
            registry_username = Ref(AWS_NO_VALUE)

        if self.model.Schemas and self.model.Schemas.RegistryPassword:
            registry_password = self.model.Schemas.RegistryPassword
        elif keyisset("RegistryPassword", schema_config):
            registry_password = schema_config["RegistryPassword"]
        else:
            registry_password = Ref(AWS_NO_VALUE)

        if self.model.Schemas and self.model.Schemas.RegistryUserInfo:
            registry_userinfo = self.model.Schemas.RegistryUserInfo
        elif keyisset("RegistryUserInfo", schema_config):
            registry_userinfo = schema_config["RegistryUserInfo"]
        else:
            registry_userinfo = Ref(AWS_NO_VALUE)

        if registry_userinfo and not isinstance(registry_userinfo, Ref):
            registry_username = Ref(AWS_NO_VALUE)
            registry_password = Ref(AWS_NO_VALUE)

        if schema_definition.Key:
            self.add_attribute_schema(
                topic_name,
                registry_url,
                registry_username,
                registry_password,
                registry_userinfo,
                schema_class,
                schema_definition.Key,
                "key",
            )
        if schema_definition.Value:
            self.add_attribute_schema(
                topic_name,
                registry_url,
                registry_username,
                registry_password,
                registry_userinfo,
                schema_class,
                schema_definition.Value,
                "value",
            )

    def render_topics(self):
        if not self.model.Topics or not self.model.Topics.Topics:
            return
        function_name = None
        if self.model.Topics.FunctionName:
            self.topic_class = CTopic
            function_name = (
                function_name
                if self.model.Topics.FunctionName.startswith("arn:aws")
                else Sub(
                    "arn:${AWS::Partition}:lambda:${AWS::Region}:${AWS::AccountId}:function:"
                    f"{self.model.Topics.FunctionName}"
                )
            )
        for topic in self.model.Topics.Topics:
            topic_cfg = topic.dict()
            if function_name:
                topic_cfg.update({"ServiceToken": function_name})
            topic_cfg.update(self.globals_config)
            topic_cfg.update(
                {
                    "ReplicationFactor": self.model.Topics.ReplicationFactor.__root__
                    if not topic.ReplicationFactor
                    else topic.ReplicationFactor,
                }
            )
            if not keyisset("Settings", topic_cfg) and keypresent(
                "Settings", topic_cfg
            ):
                del topic_cfg["Settings"]
            topic_title_raw = topic.Name.__root__
            topic_title = topic_title_raw.replace("-", "").title()
            topic_title = NONALPHANUM.sub("", topic_title)
            if topic.Schema and keyisset("Schema", topic_cfg):
                self.add_topic_schema(topic_title, topic.Schema)
                del topic_cfg["Schema"]
            if keypresent("Schema", topic_cfg):
                del topic_cfg["Schema"]
            topic_r = self.template.add_resource(
                self.topic_class(
                    topic_title,
                    DeletionPolicy=DeletionPolicy[
                        self.model.Topics.DeletionPolicy.name
                    ].value,
                    **topic_cfg,
                )
            )
            self.topics_r[topic.Name.__root__] = topic_r

    def import_topic_name(self, policy):
        """
        Method to identify whether the resource provided is a topic referenced in the template

        :param Policy policy:
        :return:
        """
        if policy.ResourceType.value == ResourceType.TOPIC.name:
            topic_name = policy.Resource
            if topic_name in self.topics_r.keys():
                return GetAtt(self.topics_r[policy.Resource], "Name")
        return policy.Resource

    def render_acls(self):
        if not self.model.ACLs or not self.model.ACLs.Policies:
            return
        function_name = None
        if self.model.ACLs.FunctionName:
            self.acl_class = CACLs
            function_name = (
                function_name
                if self.model.ACLs.FunctionName.startswith("arn:aws")
                else Sub(
                    "arn:${AWS::Partition}:lambda:${AWS::Region}:${AWS::AccountId}:function:"
                    f"{self.model.ACLs.FunctionName}"
                )
            )
        acl = {}
        acl.update(self.globals_config)
        if function_name:
            acl.update({"ServiceToken": function_name})

        policies = []
        for policy in self.model.ACLs.Policies.__root__:
            if isinstance(policy.PatternType, str):
                pattern_key = policy.PatternType
            else:
                pattern_key = policy.PatternType.name
            policies.append(
                KafkaAclPolicy(
                    Resource=self.import_topic_name(policy),
                    ResourceType=ResourceType[policy.ResourceType.name].value,
                    Principal=policy.Principal,
                    PatternType=PatternType[pattern_key].value,
                    Action=Action[policy.Action.name].value,
                    Effect=Effect[policy.Effect.name].value,
                    Host=policy.Host if policy.Host else r"*",
                )
            )
        self.template.add_resource(
            self.acl_class("ACLs", DeletionPolicy="Retain", Policies=policies, **acl)
        )



#   -*- coding: utf-8 -*-
#  SPDX-License-Identifier: MPL-2.0
#  Copyright 2020-2021 John Mille <john@compose-x.io>

import logging as logthings
import sys
from os import environ

from compose_x_common.compose_x_common import keyisset


def setup_logging():
    """Function to setup logging for ECS ComposeX.
    In case this is used in a Lambda function, removes the AWS Lambda default log handler

    :returns: the_logger
    :rtype: Logger
    """
    level = environ.get("LOGLEVEL")
    default_level = True
    formats = {
        "INFO": logthings.Formatter(
            "%(asctime)s [%(levelname)s], %(message)s",
            "%Y-%m-%d %H:%M:%S",
        ),
        "DEBUG": logthings.Formatter(
            "%(asctime)s [%(levelname)s], %(filename)s.%(lineno)d , %(funcName)s, %(message)s",
            "%Y-%m-%d %H:%M:%S",
        ),
    }

    if level is not None and isinstance(level, str):
        print("SETTING TO", level.upper())
        logthings.basicConfig(level=level.upper())
        default_level = False
    else:
        logthings.basicConfig(level="INFO")

    root_logger = logthings.getLogger()
    for h in root_logger.handlers:
        root_logger.removeHandler(h)
    the_logger = logthings.getLogger("EcsComposeX")

    if not the_logger.handlers:
        if default_level:
            formatter = formats["INFO"]
        elif keyisset(level.upper(), formats):
            formatter = formats[level.upper()]
        else:
            formatter = formats["DEBUG"]
        handler = logthings.StreamHandler(sys.stdout)
        handler.setFormatter(formatter)
        the_logger.addHandler(handler)

    return the_logger



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2021 John Mille <john@ews-network.net>

"""Top-level package for Kafka::Topic."""

from troposphere import AWSProperty

__author__ = """John Mille"""
__email__ = "john@ews-network.net"
__version__ = "0.2.0"


class KafkaAclPolicy(AWSProperty):
    """
    Class to represent a policy for EWS::Kafka::ACL.Policies
    """

    props = {
        "Resource": (str, True),
        "PatternType": (str, False),
        "Principal": (str, True),
        "ResourceType": (str, True),
        "Action": (str, True),
        "Effect": (str, True),
        "Host": (str, False),
    }



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2021 John Mille<john@ews-network.net>

"""Module to handle the custom resource for Schemas"""

import logging

from aws_cfn_custom_resource_resolve_parser import handle
from cfn_resource_provider import ResourceProvider
from kafka_schema_registry_admin.kafka_schema_registry_admin import SchemaRegistry

from cfn_kafka_admin.models.admin import EwsKafkaSchema

LOG = logging.getLogger(__name__)
LOG.setLevel(logging.INFO)


class KafkaSchema(ResourceProvider):
    def __init__(self):
        """
        Init method
        """
        self.cluster_info = {}
        super(KafkaSchema, self).__init__()
        self.request_schema = EwsKafkaSchema.schema()

    def try_replace_from_secret(self, param):
        if (
            isinstance(self.get(param), str)
            and self.get(param).find("resolve:secretsmanager") >= 0
        ):
            try:
                return handle(self.get(param))
            except Exception as error:
                LOG.error("Failed to import secrets from SecretsManager")
                self.fail(str(error))
        else:
            return self.get(param)

    def set_registry(self):
        if self.get("RegistryUserInfo"):
            user_info = self.try_replace_from_secret("RegistryUserInfo")
            username = user_info.split(":")[0]
            password = user_info.split(":")[1]
        else:
            username = self.try_replace_from_secret("RegistryUsername")
            password = self.try_replace_from_secret("RegistryPassword")
        registry_url = self.try_replace_from_secret("RegistryUrl")
        LOG.info(registry_url)
        registry = SchemaRegistry(
            **{
                "SchemaRegistryUrl": registry_url,
                "Username": username,
                "Password": password,
            }
        )
        return registry

    def create(self):
        """
        Creates a new Schema / Version in the Schema Registry
        """
        try:
            registry = self.set_registry()
            subject = self.get("Subject")
            serializer = self.get("Serializer")
            compatibility = self.get("CompatibilityMode")
            schema_def = self.get("Definition")
            schema = registry.post_subject_version(
                subject, schema_def, schema_type=serializer
            )
            registry.put_compatibility_subject_config(
                subject_name=subject, compatibility=compatibility
            )
            self.set_attribute("Id", schema["id"])
            self.physical_resource_id = subject
            self.success("Schema version created")
        except Exception as error:
            self.physical_resource_id = "could-not-create"
            self.fail(str(error))

    def update(self):
        """
        Updates the schema definition to create a new version. First, checks that the new definition is compatible.
        """
        try:
            registry = self.set_registry()
            subject = self.get("Subject")
            serializer = self.get("Serializer")
            schema_def = self.get("Definition")
            compatible = registry.post_compatibility_subjects_versions(
                subject_name=subject,
                version_id="latest",
                definition=schema_def,
                definition_type=serializer,
                as_bool=True,
            )
            if not compatible:
                print(schema_def)
                self.fail(
                    f"Schema for {subject} is not compatible with the latest version"
                )
            schema = registry.post_subject_version(subject, schema_def)
            self.set_attribute("Id", schema["id"])
            self.success("New schema version created")
        except Exception as error:
            self.fail(str(error))

    def delete(self):
        """
        Deletes the schema subject from the registry
        """
        LOG.info(self.get("Id"))
        LOG.info(self.physical_resource_id)
        if (
            self.get("Id") is None
            and self.physical_resource_id
            and self.physical_resource_id.startswith("could-not-create")
        ):
            LOG.warning("Deleting failed create resource. Nothing to do")
            self.success("Deleting failed create resource. Nothing to do")
            return
        try:
            registry = self.set_registry()
            subject = self.get("Subject")
            registry.delete_subject(subject, permanent=False)
            self.success("Schema successfully deleted")
        except Exception as error:
            self.fail(str(error))


def lambda_handler(event, context):
    provider = KafkaSchema()
    provider.handle(event, context)


if __name__ == "__main__":
    print(KafkaSchema().request_schema)



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2021 John Mille<john@ews-network.net>

"""
Module to handle Kafka topics management.
"""

from kafka import KafkaConsumer, errors
from kafka.admin import KafkaAdminClient, NewPartitions, NewTopic


def create_new_kafka_topic(
    name,
    partitions,
    cluster_info,
    replication_factor=None,
    settings=None,
):
    """
    Function to create new Kafka topic

    :param str name:
    :param int partitions:
    :param dict cluster_info: Dictionary with the Kafka information
    :param int replication_factor: Replication factor. Defaults to 3
    :param dict settings: Additional topics settings
    """
    if not replication_factor:
        replication_factor = 1
    try:
        admin_client = KafkaAdminClient(**cluster_info)
        topic = NewTopic(name, partitions, replication_factor, topic_configs=settings)
        admin_client.create_topics([topic])
        return name
    except errors.TopicAlreadyExistsError:
        raise errors.TopicAlreadyExistsError(f"Topic {name} already exists")


def delete_topic(name, cluster_info):
    """
    Function to delete kafka topic

    :param name: name of the topic to delete
    :param cluster_info: cluster information
    :return:
    """
    print(f"Deleting topic {name}")
    admin_client = KafkaAdminClient(**cluster_info)
    admin_client.delete_topics([name])


def update_kafka_topic(name, partitions, cluster_info):
    """
    Function to update existing Kafka topic

    :param name:
    :param partitions:
    :param cluster_info:
    :return:
    """
    consumer_client = KafkaConsumer(**cluster_info)
    curr_partitions = consumer_client.partitions_for_topic(name)
    if curr_partitions:
        curr_partitions_count = len(curr_partitions)
    else:
        raise LookupError(
            f"Failed to retrieve the current number of partitions for {name}"
        )
    if partitions < curr_partitions_count:
        raise ValueError(
            f"The number of partitions set {partitions} for topic "
            f"{name} is lower than current partitions count {curr_partitions} - {curr_partitions_count}"
        )
    elif partitions > curr_partitions_count:
        admin_client = KafkaAdminClient(**cluster_info)
        admin_client.create_partitions({name: NewPartitions(partitions)})
    elif partitions == curr_partitions_count:
        print(
            f"Topic {name} partitions is already set to {curr_partitions} {curr_partitions_count}. Nothing to update"
        )



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2021 John Mille<john@ews-network.net>

"""Main module."""
import uuid

from aws_cfn_custom_resource_resolve_parser import handle
from cfn_resource_provider import ResourceProvider
from compose_x_common.compose_x_common import keyisset, keypresent

from cfn_kafka_admin.common import setup_logging
from cfn_kafka_admin.kafka.acls_management import (
    create_new_acls,
    delete_acls,
    differentiate_old_new_acls,
)
from cfn_kafka_admin.models.admin import EwsKafkaAcl

LOG = setup_logging()


class KafkaACL(ResourceProvider):
    def __init__(self):
        """
        Init method
        """
        self.cluster_info = {}
        super(KafkaACL, self).__init__()
        self.request_schema = EwsKafkaAcl.schema()

    def convert_property_types(self):
        int_props = []
        boolean_props = []
        for prop in int_props:
            if keypresent(prop, self.properties) and isinstance(
                self.properties[prop], str
            ):
                self.properties[prop] = int(self.properties[prop])
        for prop in boolean_props:
            if keypresent(prop, self.properties) and isinstance(
                self.properties[prop], str
            ):
                self.properties[prop] = self.properties[prop].lower() == "true"

    def define_cluster_info(self):
        """
        Method to define the cluster information into a simple format
        """
        try:
            self.cluster_info["bootstrap_servers"] = self.get("BootstrapServers")
            self.cluster_info["security_protocol"] = self.get("SecurityProtocol")
            self.cluster_info["sasl_mechanism"] = self.get("SASLMechanism")
            self.cluster_info["sasl_plain_username"] = self.get("SASLUsername")
            self.cluster_info["sasl_plain_password"] = self.get("SASLPassword")
        except Exception as error:
            self.fail(f"Failed to get cluster information - {str(error)}")

        for key, value in self.cluster_info.items():
            if isinstance(value, str) and value.find("resolve:secretsmanager") >= 0:
                try:
                    self.cluster_info[key] = handle(value)
                except Exception as error:
                    LOG.error("Failed to import secrets from SecretsManager")
                    self.fail(str(error))

    def create(self):
        """
        Method to create a new Kafka topic
        :return:
        """
        try:
            self.define_cluster_info()
            LOG.info(f"Connecting to {self.cluster_info['bootstrap_servers']}")
            LOG.info(f"Attempting to create new ACLs {self.get('Name')}")
            topic_name = create_new_acls(
                self.get("Policies"),
                self.cluster_info,
            )
            self.physical_resource_id = str(uuid.uuid4())
            self.set_attribute("Id", self.physical_resource_id)
            self.success(f"Created new ACLs {topic_name}")
        except Exception as error:
            self.physical_resource_id = "could-not-create"
            self.fail(f"Failed to create the ACLs. {str(error)}")

    def update(self):
        """
        :return:
        """
        try:
            self.define_cluster_info()
        except Exception as error:
            self.fail(str(error))
        old_policies = self.get_old("Policies")
        for policy in old_policies:
            if not keyisset("Host", policy):
                policy.update({"Host": "*"})
        new_policies = self.get("Policies")
        acls = differentiate_old_new_acls(new_policies, old_policies)
        LOG.info("ACLs deletion")
        LOG.info(acls[1])
        LOG.info("ACLs set")
        LOG.info(acls[0])
        try:
            delete_acls(acls[1], self.cluster_info)
        except Exception as error:
            LOG.error("Failed to delete old ACLs - Moving on")
            LOG.error(error)
            LOG.error(acls[1])
        try:
            create_new_acls(acls[0], self.cluster_info)
            self.success()
            LOG.info("Successfully created new ACLs")
        except Exception as error:
            LOG.error(error)
            LOG.error("Failed to create new ACLs")
            self.fail(str(error))

    def delete(self):
        """
        Method to delete the Topic resource
        :return:
        """
        try:
            self.define_cluster_info()
            delete_acls(self.get("Policies"), self.cluster_info)
            self.success("ACLs deleted")
        except Exception as error:
            self.fail(
                f"Failed to delete topic {self.get_attribute('Name')}. {str(error)}"
            )


def lambda_handler(event, context):
    provider = KafkaACL()
    provider.handle(event, context)


if __name__ == "__main__":
    print(KafkaACL().request_schema)



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2021 John Mille<john@ews-network.net>

"""Main module."""
import logging
from os import environ

from aws_cfn_custom_resource_resolve_parser import handle
from cfn_resource_provider import ResourceProvider
from compose_x_common.compose_x_common import keypresent
from kafka import errors

from cfn_kafka_admin.kafka.topics_management import (
    create_new_kafka_topic,
    delete_topic,
    update_kafka_topic,
)
from cfn_kafka_admin.models.admin import EwsKafkaTopic

LOG = logging.getLogger(__name__)
LOG.setLevel(logging.INFO)


class KafkaTopic(ResourceProvider):
    def __init__(self):
        """
        Init method
        """
        self.cluster_info = {}
        super(KafkaTopic, self).__init__()
        self.request_schema = EwsKafkaTopic.schema()

    def convert_property_types(self):
        int_props = ["PartitionsCount", "ReplicationFactor"]
        boolean_props = ["IsConfluentKafka"]
        for prop in int_props:
            if keypresent(prop, self.properties) and isinstance(
                self.properties[prop], str
            ):
                try:
                    self.properties[prop] = int(self.properties[prop])
                except Exception as error:
                    self.fail(
                        f"Failed to get cluster information - {prop} - {str(error)}"
                    )
        for prop in boolean_props:
            if keypresent(prop, self.properties) and isinstance(
                self.properties[prop], str
            ):
                self.properties[prop] = self.properties[prop].lower() == "true"

    def define_cluster_info(self):
        """
        Method to define the cluster information into a simple format
        """
        try:
            self.cluster_info["bootstrap_servers"] = self.get("BootstrapServers")
            self.cluster_info["security_protocol"] = self.get("SecurityProtocol")
            self.cluster_info["sasl_mechanism"] = self.get("SASLMechanism")
            self.cluster_info["sasl_plain_username"] = self.get("SASLUsername")
            self.cluster_info["sasl_plain_password"] = self.get("SASLPassword")
        except Exception as error:
            self.fail(f"Failed to get cluster information - {str(error)}")

        for key, value in self.cluster_info.items():
            if isinstance(value, str) and value.find("resolve:secretsmanager") >= 0:
                try:
                    self.cluster_info[key] = handle(value)
                except Exception as error:
                    LOG.error("Failed to import secrets from SecretsManager")
                    self.fail(str(error))

    def create(self):
        """
        Method to create a new Kafka topic
        :return:
        """
        try:
            LOG.info(f"Attempting to create new topic {self.get('Name')}")
            self.define_cluster_info()
            cluster_url = (
                self.cluster_info["bootstrap.servers"]
                if self.get("IsConfluentKafka")
                else self.cluster_info["bootstrap_servers"]
            )
            LOG.info(f"Cluster is {cluster_url}")
        except Exception as error:
            self.fail(f"Failed to initialize - {str(error)}")
        if not self.get("PartitionsCount") >= 1:
            self.fail("The number of partitions must be a strictly positive value >= 1")
        try:
            topic_name = create_new_kafka_topic(
                self.get("Name"),
                self.get("PartitionsCount"),
                self.cluster_info,
                replication_factor=self.get("ReplicationFactor"),
                settings=self.get("Settings"),
            )
            self.physical_resource_id = topic_name
            self.set_attribute("Name", self.get("Name"))
            self.set_attribute("Partitions", self.get("PartitionsCount"))
            self.set_attribute("BootstrapServers", self.get("BootstrapServers"))
            self.success(f"Created new topic {topic_name}")
        except errors.TopicAlreadyExistsError as error:
            if environ.get("FAIL_IF_ALREADY_EXISTS", None) is None:
                LOG.info(f"{self.get('Name')} - Importing existing Topic")
                self.physical_resource_id = self.get("Name")
                self.set_attribute("Name", self.get("Name"))
                self.set_attribute("Partitions", self.get("PartitionsCount"))
                self.set_attribute("BootstrapServers", self.get("BootstrapServers"))
                self.success("Existing topic imported")
            else:
                self.physical_resource_id = "could-not-create-nor-import"
                self.fail(
                    f"{self.get('Name')} - Topic already exists and import is disabled, {str(error)}"
                )
        except Exception as error:
            self.physical_resource_id = "could-not-create"
            self.fail(f"Failed to create the topic {self.get('Name')}, {str(error)}")

    def update(self):
        """
        :return:
        """
        try:
            self.define_cluster_info()
            update_kafka_topic(
                self.get("Name"),
                self.get("PartitionsCount"),
                self.cluster_info,
            )
        except Exception as error:
            self.fail(str(error))

    def delete(self):
        """
        Method to delete the Topic resource
        :return:
        """
        if (
            self.get("Name") is None
            and self.physical_resource_id
            and self.physical_resource_id.startswith("could-not-create")
        ):
            LOG.warning("Deleting failed create resource.")
            self.success("Deleting non-working resource")
            return
        try:
            self.define_cluster_info()
            delete_topic(self.get("Name"), self.cluster_info)
        except Exception as error:
            self.fail(
                f"Failed to delete topic {self.get_attribute('Name')}. {str(error)}"
            )


def lambda_handler(event, context):
    provider = KafkaTopic()
    provider.handle(event, context)


if __name__ == "__main__":
    print(KafkaTopic().request_schema)



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2021 John Mille <john@ews-network.net>

"""Console script for aws_cfn_kafka_admin_provider."""

import argparse
import pprint
import sys

from cfn_kafka_admin.cfn_kafka_admin import KafkaStack


def main():
    """Console script for aws_cfn_kafka_admin_provider."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-f",
        "--definition-file",
        dest="files_paths",
        required=True,
        help="Path to the Topics / ACLs definition files",
        action="append",
    )
    parser.add_argument(
        "--config",
        "--config-file-path",
        required=False,
        dest="config_path",
        help="Override file path to the kafka definition file",
        default=None,
    )
    parser.add_argument(
        "-o", "--output-file", dest="output_file", help="Path to file output"
    )
    parser.add_argument(
        "--format",
        dest="format",
        help="Template format",
        default="json",
        choices=["json", "yaml"],
    )
    parser.add_argument("_", nargs="*")
    args = parser.parse_args()

    stack = KafkaStack(args.files_paths, args.config_path)
    stack.render_topics()
    stack.render_acls()
    if args.output_file:
        with open(args.output_file, "w") as output_fd:
            if args.format == "yaml":
                output_fd.write(stack.template.to_yaml())
            else:
                output_fd.write(stack.template.to_json())
    else:
        if args.format == "yaml":
            print(stack.template.to_yaml())
        else:
            try:
                print(stack.template.to_json())
            except TypeError:
                pprint.pprint(stack.template.to_dict())
    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2021 John Mille<john@ews-network.net>

"""
Module to handle Kafka topics management.
"""

from kafka.admin import (
    ACL,
    ACLFilter,
    ACLOperation,
    ACLPermissionType,
    ACLResourcePatternType,
    KafkaAdminClient,
    ResourcePattern,
    ResourcePatternFilter,
    ResourceType,
)


def differentiate_old_new_acls(new_policies, old_policies):
    """
    Function to differentiate with ACLs are common and shall be kept, which ones are to be added and then removed.

    :param list new_policies:
    :param list old_policies:
    :return: the new acls and old acls
    :rtype: tuple
    """

    common_policies = [d for d in new_policies if d in old_policies]
    common_policies_set = set()
    for policy in common_policies:
        common_policies_set.add(tuple(policy.items()))
    new_policies_set = set()
    old_policies_set = set()
    for policy in new_policies:
        t_policy = tuple(policy.items())
        if t_policy not in new_policies_set or t_policy in common_policies:
            new_policies_set.add(t_policy)
    for policy in old_policies:
        t_policy = tuple(policy.items())
        if (
            t_policy not in old_policies_set
            and t_policy not in common_policies
            and t_policy not in new_policies_set
        ):
            old_policies_set.add(t_policy)
    final_delete_acls = [dict(k) for k in old_policies_set]
    final_new_acls = [dict(k) for k in new_policies_set]
    return final_new_acls, final_delete_acls


def create_new_acls(acls, cluster_info):
    """
    Function to iterate over the given ACL policies and apply them

    :param list acls:
    :param cluster_info:
    :return:
    """
    admin_client = KafkaAdminClient(**cluster_info)
    kafka_acls = []
    for policy in acls:
        if isinstance(policy, dict):
            new_acl = ACL(
                principal=policy["Principal"],
                host=policy["Host"],
                operation=ACLOperation[policy["Action"]],
                permission_type=ACLPermissionType[policy["Effect"]],
                resource_pattern=ResourcePattern(
                    resource_type=ResourceType[policy["ResourceType"]],
                    resource_name=policy["Resource"],
                    pattern_type=ACLResourcePatternType[policy["PatternType"]],
                ),
            )
            kafka_acls.append(new_acl)
    admin_client.create_acls(kafka_acls)


def delete_acls(acls, cluster_info):
    """
    Function to delete the ACLs.
    :param acls:
    :param cluster_info:
    :return:
    """
    admin_client = KafkaAdminClient(**cluster_info)
    policies = []
    for policy in acls:
        new_filter = ACLFilter(
            principal=policy["Principal"],
            host=policy["Host"] if "Host" in policy else "*",
            operation=ACLOperation[policy["Action"]],
            permission_type=ACLPermissionType[policy["Effect"]],
            resource_pattern=ResourcePatternFilter(
                resource_type=ResourceType[policy["ResourceType"]],
                resource_name=policy["Resource"],
                pattern_type=ACLResourcePatternType[policy["PatternType"]],
            ),
        )
        policies.append(new_filter)
    admin_client.delete_acls(policies)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cfn_kafka_admin',
 'cfn_kafka_admin.cfn_resources_definitions',
 'cfn_kafka_admin.kafka',
 'cfn_kafka_admin.lambda_functions',
 'cfn_kafka_admin.models',
 'cfn_kafka_admin.specs']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'aws-cfn-custom-resource-resolve-parser>=0.2.1,<0.3.0',
 'cfn-resource-provider>=1.0.7,<2.0.0',
 'compose-x-common[aws]>=0.2.2,<0.3.0',
 'datamodel-code-generator[http]>=0.11.14,<0.12.0',
 'importlib-resources>=5.3.0,<6.0.0',
 'jsonschema>=4.1.2,<5.0.0',
 'kafka-python>=2.0.2,<3.0.0',
 'kafka-schema-registry-admin>=0.1,<0.2',
 'troposphere>=3.1.0,<4.0.0']

entry_points = \
{'console_scripts': ['aws-cfn-kafka-admin-provider = cfn_kafka_admin.cli:main']}

setup_kwargs = {
    'name': 'cfn-kafka-admin',
    'version': '0.2.0',
    'description': 'AWS CloudFormation Resources to manage Kafka',
    'long_description': '===============\ncfn-kafka-admin\n===============\n\n------------------------------------------------------------------------------\nCLI Tool and Lambda Functions to CRUD Kafka resources via AWS CloudFormation\n------------------------------------------------------------------------------\n\n\n|PYPI_VERSION|\n\n|FOSSA| |PYPI_LICENSE|\n\n|CODE_STYLE| |TDD|\n\n.. image:: https://codebuild.eu-west-1.amazonaws.com/badges?uuid=eyJlbmNyeXB0ZWREYXRhIjoiY2xwc0NER1JuU1J3MThYczhFMDJLWlQxWGpoRnhNWHNtbGN1NGpVMVNTMk12UlQxdWVlZ2w5YnhPQzhkMnV4cTI0S0tIdTRyTmRHWWErWXJPNWFpcWlzPSIsIml2UGFyYW1ldGVyU3BlYyI6IkxaRGZCMW1KbVE1RWRJYjciLCJtYXRlcmlhbFNldFNlcmlhbCI6MX0%3D&branch=main\n        :target: https://eu-west-1.codebuild.aws.amazon.com/project/eyJlbmNyeXB0ZWREYXRhIjoibVAvWVBBNjZlNWFwTWEwSEdWcGx6MWpudy9KeEZTb1lXdWFuQ3FwbjJCRTBnc1lyZm41eHRqV2k0bDN6UTBmaEpJMGd0Y3I3Vm5kTGtZQzc1b25Uckxxd3hERzlpSzJndVFOekJUR0NMM0V0YXljSWx4Yjc2YmJpUzlZM01RPT0iLCJpdlBhcmFtZXRlclNwZWMiOiI3bnllb1dlbU8rZis1ekh5IiwibWF0ZXJpYWxTZXRTZXJpYWwiOjF9\n\n\nManage Kafka resources via AWS CFN\n===================================\n\n* Topics\n* ACLs\n* Schemas (non AWS Glue Schema)\n\n\n* Free software: MPL-2.0\n* Documentation: https://cfn-kafka-admin.readthedocs.io.\n\n.. |PYPI_VERSION| image:: https://img.shields.io/pypi/v/cfn-kafka-admin.svg\n        :target: https://pypi.python.org/pypi/cfn-kafka-admin\n\n.. |PYPI_LICENSE| image:: https://img.shields.io/pypi/l/cfn-kafka-admin\n    :alt: PyPI - License\n    :target: https://github.com/compose-x/cfn-kafka-admin/blob/master/LICENSE\n\n.. |PYPI_PYVERS| image:: https://img.shields.io/pypi/pyversions/cfn-kafka-admin\n    :alt: PyPI - Python Version\n    :target: https://pypi.python.org/pypi/cfn-kafka-admin\n\n.. |PYPI_WHEEL| image:: https://img.shields.io/pypi/wheel/cfn-kafka-admin\n    :alt: PyPI - Wheel\n    :target: https://pypi.python.org/pypi/cfn-kafka-admin\n\n.. |FOSSA| image:: https://app.fossa.com/api/projects/git%2Bgithub.com%2Fcompose-x%2Fcfn-kafka-admin.svg?type=shield\n\n.. |CODE_STYLE| image:: https://img.shields.io/badge/codestyle-black-black\n    :alt: CodeStyle\n    :target: https://pypi.org/project/black/\n\n.. |TDD| image:: https://img.shields.io/badge/tdd-pytest-black\n    :alt: TDD with pytest\n    :target: https://docs.pytest.org/en/latest/contents.html\n\n.. |BDD| image:: https://img.shields.io/badge/bdd-behave-black\n    :alt: BDD with Behave\n    :target: https://behave.readthedocs.io/en/latest/\n',
    'author': 'johnpreston',
    'author_email': 'john@compose-x.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#  -*- coding: utf-8 -*-
# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2021 John Mille <john@ews-network.net>

"""Definition of EWS::Kafka::Topic resource."""

from troposphere import AWSObject
from troposphere.validators import positive_integer

from cfn_kafka_admin.cfn_resources_definitions import KafkaAclPolicy


class KafkaTopic(AWSObject):
    """
    Class to represent EWS::Kafka::Topic
    """

    resource_type = "EWS::Kafka::Topic"
    props = {
        "BootstrapServers": (str, True),
        "ReplicationFactor": (positive_integer, False),
        "SecurityProtocol": (str, False),
        "SASLMechanism": (str, False),
        "SASLUsername": (str, False),
        "SASLPassword": (str, False),
        "Name": (str, True),
        "PartitionsCount": (positive_integer, True),
        "Settings": (dict, False),
    }


class KafkaAcl(AWSObject):
    """
    Class to represent EWS::Kafka::ACL
    """

    resource_type = "EWS::Kafka::ACL"
    props = {
        "BootstrapServers": (str, True),
        "ReplicationFactor": (positive_integer, False),
        "SecurityProtocol": (str, False),
        "SASLMechanism": (str, False),
        "SASLUsername": (str, False),
        "SASLPassword": (str, False),
        "Policies": ([KafkaAclPolicy], True),
    }


class KafkaTopicSchema(AWSObject):
    """
    Class to represent EWS::Kafka::Schema
    """

    resource_type = "EWS::Kafka::Schema"
    props = {
        "RegistryUrl": (str, True),
        "RegistryUsername": (str, False),
        "RegistryPassword": (str, False),
        "RegistryUserInfo": (str, False),
        "Subject": (str, True),
        "Type": (str, True),
        "Definition": ((str, dict), True),
        "SerializeAttribute": (str, True),
        "CompatibilityMode": (str, True),
    }
